__version__ = "1.0.0"

# preserved here for legacy reasons
__model_version__ = "latest"

import higgs.audiotools as audiotools

audiotools.ml.BaseModel.INTERN += ["dac.**"]
audiotools.ml.BaseModel.EXTERN += ["einops"]


from . import nn
from . import model
from . import utils
from .model import DAC
from .model import DACFile
